@@ -1,5 +1,5 @@
 /* Intellimouse Wheel Thinger Configuration Helper
- * VERSION 0.9.5
+ * VERSION 0.9.6
  * Development release
  * Copylefted under the GNU Public License
  * No code here may be used for profit without the permission of the author.
@@ -151,7 +151,7 @@ int handle_button(Jax *jax)
 	}
 	else
 	{
-		if(jax->xe.xbutton.x<MAX_SCALE_W)
+		if(jax->xe.xbutton.button==1 && jax->xe.xbutton.x<MAX_SCALE_W)
 		{
 			drawBox(jax,0,0,MAX_SCALE_W,MAX_SCALE_H,grays[0].pixel,grays[2].pixel);
 			drawBox(jax,1,1,MAX_SCALE_W-2,MAX_SCALE_H-2,grays[0].pixel,grays[2].pixel);
@@ -413,12 +413,30 @@ int GrabWheel(Jax *jax,Hotspot *hs)
 	XEvent e;
 	Hotspot *hsp;
 	char *str;
-	
+	char *wheelstr[]=
+	{
+		"button: Left",
+		"button: Middle",
+		"button: Right",
+		"wheel: Up",
+		"wheel: Down",
+		"wheel: Left",
+		"wheel: Right"
+	};
+
 	if(!useFifo && !grabbed)
-		grabButtons(jax->d);
-	else
+	{
+		grabButtons(jax->d,0);
+		Printf("grabbed=%d\n",(grabbed?"True":"False"));
+	}
+	if(useFifo)
 		openFifo();
-	while(!getInput(jax->d,&e,&xmk,km,&button));
+	do
+		getInput(jax->d,&e,&xmk,km,&button);
+	while((button<4 && grabbed) || e.type!=ButtonPress);
+	Printf("button=%d\n",button);
+	if(!useFifo && grabbed)
+		ungrabButtons(jax->d,0);
 	hsp=getHS(grabw_hs[0]);
 	jax->xe.xexpose.count=-1;
 	jax->xe.xexpose.x=hsp->x;
@@ -428,7 +446,7 @@ int GrabWheel(Jax *jax,Hotspot *hs)
 	hsp=getHS(grabw_hs[0]);
 	if(hsp->label)
 		free(hsp->label);
-	hsp->label=strdup(button==4?"wheel: Up":"wheel: Down");
+	hsp->label=strdup(wheelstr[(int)button-1]);
 	jax->xe.xexpose.height+=hsp->h;
 	hsi++;
 	for(hsi=1,i=0;hsi<numgrabw && i<32*8;i++)
@@ -452,8 +470,6 @@ int GrabWheel(Jax *jax,Hotspot *hs)
 		hsi++;
 	}
 	handle_expose(jax);
-	if(!useFifo && grabbed)
-		ungrabButtons(jax->d);
 	freeAllX(NULL,&xmk,NULL);
 	return(False);
 }
@@ -600,6 +616,8 @@ int HSeventhandler(Jax *jax)
 		switch(jax->xe.xany.type)
 		{
 			case ButtonPress:
+				if(jax->xe.xbutton.button!=1)
+					return(0);
 				if(inHS(&hs[i],jax->xe.xbutton.x, jax->xe.xbutton.y))
 				{
 					doHS(jax,&hs[i]);
@@ -607,6 +625,8 @@ int HSeventhandler(Jax *jax)
 				}
 				break;
 			case ButtonRelease:
+				if(jax->xe.xbutton.button!=1)
+					return(0);
 				if(inHS(&hs[i],jax->xe.xbutton.x, jax->xe.xbutton.y))
 					return(doHS(jax,&hs[i]));
 				else
@@ -654,7 +674,7 @@ int doHS(Jax *jax, Hotspot *h)
 				case BUTTON:
 					if(h->state&ACTIVE)
 					{
-						if(inHS(h, jax->xe.xmotion.x, jax->xe.xmotion.y))
+						if(inHS(h, jax->xe.xbutton.x, jax->xe.xbutton.y))
 						{
 							if(h->state&REST)
 								h->state=DIRTY|ACTIVE;
@@ -674,7 +694,7 @@ int doHS(Jax *jax, Hotspot *h)
 			switch(h->type)
 			{
 				case BUTTON:
-					if(inHS(h, jax->xe.xmotion.x, jax->xe.xmotion.y))
+					if(inHS(h, jax->xe.xbutton.x, jax->xe.xbutton.y))
 						err=h->buttonfunc(jax,h);
 					if(!(h->state&REST))
 						h->state=REST|DIRTY;
