@@ -1,5 +1,5 @@
 /* IMWheel Header File for more private functions
- * VERSION 0.9.5
+ * VERSION 0.9.6
  * Development release
  * Copylefted under the Linux GNU Public License
  * No code here may be used for profit without the permission of the author.
@@ -10,14 +10,19 @@
 #define IMWHEEL_H
 
 char getInput(Display *d, XEvent *e, XModifierKeymap **xmk, char km[32], char *button);
-void grabButtons(Display *d);
-void ungrabButtons(Display *d);
+void grabButtons(Display *d, Window w);
+void ungrabButtons(Display *d, Window w);
 void freeAllX(XClassHint *xch, XModifierKeymap **xmk, char **wname);
+typedef struct
+{
+	char x,y;
+} Stick;
 
 extern const char *optionusage[][2];
 extern Bool grabbed;
-extern int buttonFlip, useFifo, detach, quit, pid_check;
+extern int buttonFlip, useFifo, detach, quit, pid_check, restart, threshhold;
 extern int fifofd;
-extern char *fifoName;
+extern char *fifoName, *displayName;
+extern Stick stick;
 
 #endif
