@@ -1,5 +1,5 @@
 /* Intellimouse Wheel Thinger Utilities
- * VERSION 0.9.5
+ * VERSION 0.9.6
  * Development release
  * Copylefted under the GNU Public License
  * No code here may be used for profit without the permission of the author.
@@ -29,7 +29,7 @@
 #undef UTIL_C
 #include "imwheel.h"
 
-int buttons[NUM_BUTTONS]={Button4, Button5};
+int buttons[NUM_BUTTONS]={Button4, Button5, Button4+2, Button5+2};
 int statebits[STATE_MASK+1];
 char *wname=NULL;
 XClassHint xch={NULL,NULL};
@@ -83,6 +83,26 @@ const char *keys[NUM_BUTTONS][1<<NUM_STATES]=
 		"Right",		//ShiftMask|            Mod1Mask
 		"Right",		//          ControlMask|Mod1Mask
 		"Right"			//ShiftMask|ControlMask|Mod1Mask
+	},
+	{
+		"Left",                 //None
+		"Home",                 //ShiftMask
+		"Left",                 //          ControlMask
+		"Left",                 //ShiftMask|ControlMask
+		"Left",                 //                      Mod1Mask
+		"Left",                 //ShiftMask|            Mod1Mask
+		"Left",                 //          ControlMask|Mod1Mask
+		"Left"                  //ShiftMask|ControlMask|Mod1Mask
+	},
+	{
+		"Right",                //None
+		"End",                  //ShiftMask
+		"Right",                //          ControlMask
+		"Right",                //ShiftMask|ControlMask
+		"Right",                //                      Mod1Mask
+		"Right",                //ShiftMask|            Mod1Mask
+		"Right",                //          ControlMask|Mod1Mask
+		"Right"                 //ShiftMask|ControlMask|Mod1Mask
 	}
 };
 
@@ -111,26 +131,30 @@ void exitParent(int num)
 
 void getOptions(int argc, char **argv, char *opts, const struct option *options)
 {
-	int ch,i,killold=False,restart=False;
+	int ch,i,killold=False,invalidOpts=False;
 
-	while((ch=getopt_long(argc,argv,opts,options,&i))>=0)
+	while((ch=getopt_long_only(argc,argv,opts,options,&i))>=0)
 	{
+		Printf("ch=%c",ch);
+		if(optarg)
+			Printf("\t=%s",optarg);
+		//Printf(" options[%d].name=%s",options[i].name);
 		switch(ch)
 		{
-			case 0:
-				printf("ch=%c\n\toption[%d]=\"%s\"\n",ch,i,options[i].name);
+			/*case 0:
+				Printf("\toption[%d]=\"%s\"",ch,i,options[i].name);
 				if(optarg)
-					printf("\toptarg=\"%s\"\n",optarg);
+					Printf("\toptarg=\"%s\"",optarg);*/
 			case '4':
 				buttonFlip=True;
 				break;
+			case 'X':
+				displayName=strdup(optarg);
+				break;
 			case 'W':
 				useFifo=True;
 				if(optarg)
-				{
-					fifoName=malloc(strlen(optarg)+1);
-					strcpy(fifoName,optarg);
-				}
+					fifoName=strdup(optarg);
 				break;
 			case 'd':
 				detach=False;
@@ -150,18 +174,25 @@ void getOptions(int argc, char **argv, char *opts, const struct option *options)
 			case 'R':
 				restart=True;
 				break;
+			case 't':
+				threshhold=atoi(optarg);
+				break;
 			case 'h':
 			case '?':
 				printUsage(argv[0],options,optionusage);
 				exit(0);
 				break;
 			default:
-				printf("option -%c is unknown!\n",ch);
+				fprintf(stderr,"imwheel: Option %s is unknown!\n",argv[optind]);
 		}
+		Printf("\n");
 	}
+	if(invalidOpts)
+		exit(1);
 	if(!restart)
 	{
-		if(killold)
+		signal(SIGINT,exitParent);
+		if(killold && useFifo)
 			KillIMWheel();
 		if(quit)
 			exit(0);
@@ -177,10 +208,10 @@ void getOptions(int argc, char **argv, char *opts, const struct option *options)
 		else
 			fprintf(stderr,"imwheel is not running as a daemon.\n");
 		setsid();
-		if(pid_check)
+		if(pid_check && useFifo)
 			WritePID();
 		else
-			fprintf(stderr,"imwheel is not checking/writing a pid file, BE CAREFUL!\nAn imwheel may be running already, two or more imwheel processes\nwill not operate as expected!\n");
+			fprintf(stderr,"imwheel is not checking/writing a pid file, BE CAREFUL!\nAn imwheel may be running already, two or more imwheel processes\non the same X display, or using gpm -W, will not operate as expected!\n");
 		if(detach)
 			kill(getppid(),SIGUSR1);
 		fprintf(stderr,"imwheel started (pid=%d)\n",getpid());
@@ -225,6 +256,7 @@ void KillIMWheel()
 {
 	FILE *f;
 	pid_t pid;
+	char procfname[80],procname[1024];
 
 	if((f=fopen(PIDFILE,"r")))
 	{
@@ -233,10 +265,24 @@ void KillIMWheel()
 		fclose(f);
 		if(pid>1)
 		{
-			if(killpg(pid,SIGTERM)<0)
-				perror("imwheel not killed: ");
+			sprintf(procfname,"/proc/%d/status",pid);
+			if((f=fopen(procfname,"r")))
+			{
+				fscanf(f,"Name:\t%s",procname);
+				fclose(f);
+				fprintf(stderr,"pid %d is %s.\n",pid,procname);
+				if(!strcmp(procname,"imwheel"))
+				{
+					if(killpg(pid,SIGTERM)<0)
+						perror("imwheel not killed: ");
+					else
+						fprintf(stderr,"imwheel(pid=%d) killed.\n",pid);
+				}
+				else
+					fprintf(stderr,"pid %d is not imwheel, not killing.\n",pid);
+			}
 			else
-				fprintf(stderr,"imwheel(pid=%d) killed.\n",pid);
+				perror("imwheel process could not be verified: ");
 		}
 		if(unlink(PIDFILE)<0)
 			perror("imwheel pid file could not be removed: ");
@@ -289,8 +335,8 @@ void setbit(char *buf, int n, Bool val)
 
 int getbit(char *buf, int n)
 {
-	if(buf[n/8]&(1<<(n%8)))
-		Printf("getbit:n=%d,%d\n",n/8,n%8);
+	/*if(debug && buf[n/8]&(1<<(n%8)))
+		Printf("getbit:n=%d,%d\n",n/8,n%8);*/
 	return(buf[n/8]&(1<<(n%8)));
 }
 
@@ -343,7 +389,8 @@ void printUsage(char *pname, const struct option options[], const char *usage[][
 	int i,maxa=0,maxb=0,len;
 	char str[80];
 
-	printf("%s:",pname);
+	printf("imwheel %s by -=<Long Island Man>=- <jcatki@mysolution.com>\n",VERSION);
+	printf("%s",pname);
 	for(i=0;options[i].name;i++)
 	{
 		len=0;
@@ -394,7 +441,7 @@ void printUsage(char *pname, const struct option options[], const char *usage[][
 		{
 			len=strlen(usage[i][1]);
 			if(maxa+maxb+len+2>80)
-				printf("\n%80.80s%s","",usage[i][1]);
+				printf("\n%80.80s",usage[i][1]);
 			else
 				printf(" %s",usage[i][1]);
 		}
@@ -591,13 +638,14 @@ struct WinAction *getRC()
 			Printf("Button: \"%s\"\n",line);
 			if(!strcasecmp(line,"Up"))
 				newwa[num_wa-1].button=4;
+			else if(!strcasecmp(line,"Down"))
+				newwa[num_wa-1].button=5;
+			else if(!strcasecmp(line,"Left"))
+				newwa[num_wa-1].button=6;
+			else if(!strcasecmp(line,"Right"))
+				newwa[num_wa-1].button=7;
 			else
-			{
-				if(!strcasecmp(line,"Down"))
-					newwa[num_wa-1].button=5;
-				else
-					exitString("Unrecognized wheel action in config.\n%s\n",line);
-			}
+				exitString("Unrecognized wheel action in config.\n%s\n",line);
 			Printf("\t=%d\n",newwa[num_wa-1].button);
 			memmove(line,p+1,strlen(p+1)+1);
 			//Get Keysym Out
@@ -709,7 +757,7 @@ void printWA(struct WinAction *wa)
 		return;
 	Printf("WinAction (%p):\n",wa);
 	Printf("\tWindow Regex     : \"%s\"\n",wa->id);
-	if(wa->button<6)
+	if(wa->button<8)
 	{
 		Printf("\tKeysyms Mask (%p):\n",wa->in);
 		for(i=0; wa->in[i]; i++)
@@ -764,7 +812,7 @@ void writeRC(struct WinAction *wa)
 			fprintf(f,"\n\"%s\"\n",p->id);
 			cur=p;
 		}
-		if(p->button<6)
+		if(p->button<8)
 		{
 			for(i=0; p->in[i]; i++)
 				fprintf(f,"%s%s",(i?"|":""),p->in[i]);
@@ -969,6 +1017,8 @@ void closeFifo()
 
 void openFifo() //also will close,flush, and reopen if fifo is already open!
 {
+	// Clear stick (reset state)
+	stick.x=stick.y=0;
 	// preflush, or else!
 	flushFifo();
 	// open for real now!
